@@ -29,7 +29,7 @@ from oslo_utils import excutils
 from kuryr.lib import constants as lib_const
 from kuryr.lib import exceptions
 from kuryr.lib import utils as lib_utils
-from kuryr.lib._i18n import _LE, _LI, _LW
+
 from kuryr_libnetwork import app
 from kuryr_libnetwork import config
 from kuryr_libnetwork import constants as const
@@ -68,7 +68,7 @@ def check_for_neutron_ext_support():
     except n_exceptions.NeutronClientException as e:
         if e.status_code == n_exceptions.NotFound.status_code:
             raise exceptions.MandatoryApiMissing(
-                            "Neutron extension with alias '{0}' not found"
+                "Neutron extension with alias '{0}' not found"
                             .format(MANDATORY_NEUTRON_EXTENSION))
 
 
@@ -84,8 +84,8 @@ def check_for_neutron_tag_support(ext_name):
     except n_exceptions.NeutronClientException as e:
         setattr(app, ext_rename, False)
         if e.status_code == n_exceptions.NotFound.status_code:
-            LOG.warning(_LW("Neutron extension %s not supported. "
-                            "Continue without using them."), ext_name)
+            LOG.warning("Neutron extension %s not supported. "
+                        "Continue without using them.", ext_name)
 
 
 def load_default_subnet_pools():
@@ -113,8 +113,8 @@ def _cache_default_subnetpool_ids(app):
                 for subnetpool in subnetpools['subnetpools']:
                     default_subnetpool_id_set.add(subnetpool['id'])
         except n_exceptions.NeutronClientException as ex:
-            LOG.error(_LE("Error happened during retrieving the default"
-                          " subnet pools: %s"), ex)
+            LOG.error("Error happened during retrieving the default"
+                      " subnet pools: %s", ex)
         app.DEFAULT_POOL_IDS = frozenset(default_subnetpool_id_set)
 
 
@@ -160,9 +160,9 @@ def _get_subnets_by_interface_cidr(neutron_network_id,
         network_id=neutron_network_id, cidr=six.text_type(iface.network))
     if len(subnets) > 2:
         raise exceptions.DuplicatedResourceException(
-                "Multiple Neutron subnets exist for the network_id={0}"
-                "and cidr={1}"
-                .format(neutron_network_id, iface.network))
+            "Multiple Neutron subnets exist for the network_id={0}"
+            "and cidr={1}"
+            .format(neutron_network_id, iface.network))
     return subnets
 
 
@@ -207,8 +207,8 @@ def _create_port(endpoint_id, neutron_network_id, interface_mac, fixed_ips):
     try:
         rcvd_port = app.neutron.create_port({'port': port})
     except n_exceptions.NeutronClientException as ex:
-        LOG.error(_LE("Error happened during creating a"
-                      " Neutron port: %s"), ex)
+        LOG.error("Error happened during creating a"
+                  " Neutron port: %s", ex)
         raise
     return rcvd_port['port']
 
@@ -305,6 +305,14 @@ def _neutron_net_remove_tags(netid, tag):
         _neutron_net_remove_tag(netid, tag)
 
 
+def _neutron_subnetpool_add_tag(poolid, tag):
+    _neutron_add_tag('subnetpools', poolid, tag)
+
+
+def _neutron_subnetpool_remove_tag(poolid, tag):
+    _neutron_remove_tag('subnetpools', poolid, tag)
+
+
 def _neutron_subnet_add_tag(subnetid, tag):
     _neutron_add_tag('subnets', subnetid, tag)
 
@@ -325,9 +333,9 @@ def _neutron_add_tag(resource_type, resource_id, tag):
     try:
         app.neutron.add_tag(resource_type, resource_id, tag)
     except n_exceptions.NotFound:
-        LOG.warning(_LW("Neutron tags extension for given "
-                        "resource type is not supported, "
-                        "cannot add tag to %s."), resource_type)
+        LOG.warning("Neutron tags extension for given "
+                    "resource type is not supported, "
+                    "cannot add tag to %s.", resource_type)
 
 
 def _neutron_remove_tag(resource_type, resource_id, tag):
@@ -353,8 +361,8 @@ def _port_active(neutron_port_id, vif_plug_timeout):
         try:
             port = app.neutron.show_port(neutron_port_id)
         except n_exceptions.NeutronClientException as ex:
-            LOG.error(_LE('Could not get the port %s to check '
-                          'its status'), ex)
+            LOG.error('Could not get the port %s to check '
+                      'its status', ex)
         else:
             if port['port']['status'] == lib_const.PORT_STATUS_ACTIVE:
                 port_active = True
@@ -381,8 +389,8 @@ def _program_expose_ports(options, port_id):
         sg_id = sg['security_group']['id']
 
     except n_exceptions.NeutronClientException as ex:
-        LOG.error(_LE("Error happened during creating a "
-                      "Neutron security group: %s"), ex)
+        LOG.error("Error happened during creating a "
+                  "Neutron security group: %s", ex)
         raise exceptions.ExportPortFailure(
             ("Could not create required security group {0} "
              "for setting up exported port ").format(sec_group))
@@ -396,7 +404,7 @@ def _program_expose_ports(options, port_id):
             proto_port_dict[proto].append(port)
         except KeyError:
             # This should not happen as Docker client catches such errors
-            LOG.error(_LE("Unrecognizable protocol %s"), proto)
+            LOG.error("Unrecognizable protocol %s", proto)
             app.neutron.delete_security_group(sg_id)
             raise exceptions.ExportPortFailure(
                 ("Bad protocol number for exposed port. Deleting "
@@ -422,9 +430,9 @@ def _program_expose_ports(options, port_id):
                 app.neutron.create_security_group_rule({'security_group_rule':
                                                         sec_group_rule})
             except n_exceptions.NeutronClientException as ex:
-                LOG.error(_LE("Error happened during creating a "
-                              "Neutron security group "
-                              "rule: %s"), ex)
+                LOG.error("Error happened during creating a "
+                          "Neutron security group "
+                          "rule: %s", ex)
                 app.neutron.delete_security_group(sg_id)
                 raise exceptions.ExportPortFailure(
                     ("Could not create required security group rules {0} "
@@ -442,8 +450,8 @@ def _program_expose_ports(options, port_id):
         app.neutron.update_port(port_id,
                                 {'port': {'security_groups': sgs}})
     except n_exceptions.NeutronClientException as ex:
-        LOG.error(_LE("Error happened during updating a "
-                      "Neutron port: %s"), ex)
+        LOG.error("Error happened during updating a "
+                  "Neutron port: %s", ex)
         app.neutron.delete_security_group(sg_id)
         raise exceptions.ExportPortFailure(
             ("Could not update port with required security groups{0} "
@@ -457,8 +465,8 @@ def _get_cidr_from_subnetpool(**kwargs):
         pool_id = pool['id']
         prefixes = pool['prefixes']
         if len(prefixes) > 1:
-            LOG.warning(_LW("More than one prefixes present. "
-                            "Picking first one."))
+            LOG.warning("More than one prefixes present. "
+                        "Picking first one.")
 
         return ipaddress.ip_network(six.text_type(prefixes[0])), pool_id
     else:
@@ -514,15 +522,15 @@ def revoke_expose_ports(port_id):
                                     {'port':
                                      {'security_groups': existing_sgs}})
         except n_exceptions.NeutronClientException as ex:
-            LOG.error(_LE("Error happened during updating a "
-                          "Neutron port with a new list of "
-                          "security groups: {0}").format(ex))
+            LOG.error("Error happened during updating a "
+                      "Neutron port with a new list of "
+                      "security groups: {0}".format(ex))
     try:
         for sg in removing_sgs:
             app.neutron.delete_security_group(sg)
     except n_exceptions.NeutronClientException as ex:
-        LOG.error(_LE("Error happened during deleting a "
-                      "Neutron security group: {0}").format(ex))
+        LOG.error("Error happened during deleting a "
+                  "Neutron security group: {0}".format(ex))
 
 
 def _create_kuryr_subnet(pool_cidr, subnet_cidr, pool_id, network_id, gateway):
@@ -541,6 +549,26 @@ def _create_kuryr_subnet(pool_cidr, subnet_cidr, pool_id, network_id, gateway):
     LOG.debug("Created kuryr subnet %s", new_kuryr_subnet)
 
 
+def _create_kuryr_subnetpool(pool_cidr):
+    pool_name = lib_utils.get_neutron_subnetpool_name(pool_cidr)
+    pools = _get_subnetpools_by_attrs(name=pool_name)
+    if len(pools):
+        raise exceptions.KuryrException(
+            "Another pool with same cidr exist. ipam and network"
+            " options not used to pass pool name")
+
+    cidr = ipaddress.ip_network(six.text_type(pool_cidr))
+    new_subnetpool = {
+        'name': pool_name,
+        'default_prefixlen': cidr.prefixlen,
+        'prefixes': [pool_cidr]}
+    LOG.info("Creating subnetpool with the given pool CIDR")
+    created_subnetpool_response = app.neutron.create_subnetpool(
+        {'subnetpool': new_subnetpool})
+    pool = created_subnetpool_response['subnetpool']
+    return pool
+
+
 @app.route('/Plugin.Activate', methods=['POST'])
 def plugin_activate():
     """Returns the list of the implemented drivers.
@@ -607,6 +635,45 @@ def network_driver_discover_delete():
     return flask.jsonify(const.SCHEMA['SUCCESS'])
 
 
+@app.route('/NetworkDriver.AllocateNetwork', methods=['POST'])
+def network_driver_allocate_network():
+    """Allocate network specific resources passing network id and network specific config.
+
+    Libnetwork miss this API in their doc.
+    https://github.com/docker/libnetwork/issues/1699
+
+    See the following like for more detail about the spec:
+
+      https://github.com/docker/libnetwork/blob/master/driverapi/driverapi.go # noqa
+    """
+    json_data = flask.request.get_json(force=True)
+    LOG.debug("Received JSON data %s for "
+              "/NetworkDriver.AllocateNetwork", json_data)
+    # Note(limao): This API will only called in docker swarm mode
+    # The returned options are passed to CreateNetwork.
+    return flask.jsonify({'Options': json_data.get('Options')})
+
+
+@app.route('/NetworkDriver.FreeNetwork', methods=['POST'])
+def network_driver_free_network():
+    """Free network specific resources associated with a given network id.
+
+    Libnetwork miss this API in their doc.
+    https://github.com/docker/libnetwork/issues/1699
+
+    See the following like for more detail about the spec:
+
+      https://github.com/docker/libnetwork/blob/master/driverapi/driverapi.go # noqa
+    """
+    json_data = flask.request.get_json(force=True)
+    LOG.debug("Received JSON data %s for "
+              "/NetworkDriver.FreeNetwork", json_data)
+    # Note(limao): This API will only called in docker swarm mode,
+    #              we do not have network resource to free right now,
+    #              so just return SUCCESS.
+    return flask.jsonify(const.SCHEMA['SUCCESS'])
+
+
 @app.route('/NetworkDriver.CreateNetwork', methods=['POST'])
 def network_driver_create_network():
     """Creates a new Neutron Network which name is the given NetworkID.
@@ -727,8 +794,8 @@ def network_driver_create_network():
         try:
             neutron_uuid = app.driver.get_default_network_id()
         except n_exceptions.NeutronClientException as ex:
-            LOG.error(_LE("Failed to retrieve the default driver "
-                          "network due to Neutron error: %s"), ex)
+            LOG.error("Failed to retrieve the default driver "
+                      "network due to Neutron error: %s", ex)
             raise
 
     if not neutron_uuid and not neutron_name:
@@ -739,8 +806,8 @@ def network_driver_create_network():
         _neutron_net_add_tags(network['network']['id'], container_net_id,
                               tags=app.tag)
 
-        LOG.info(_LI("Created a new network with name "
-                     "%(neutron_network_name)s successfully: %(network)s"),
+        LOG.info("Created a new network with name "
+                 "%(neutron_network_name)s successfully: %(network)s",
                  {'neutron_network_name': neutron_network_name,
                   'network': network})
     else:
@@ -753,25 +820,26 @@ def network_driver_create_network():
                 specified_network = neutron_name
             if not networks:
                 raise exceptions.KuryrException(
-                      ("Specified network id/name({0}) does not "
-                       "exist.").format(specified_network))
+                    ("Specified network id/name({0}) does not "
+                     "exist.").format(specified_network))
             network_id = networks[0]['id']
         except n_exceptions.NeutronClientException as ex:
-            LOG.error(_LE("Error happened during listing "
-                          "Neutron networks: %s"), ex)
+            LOG.error("Error happened during listing "
+                      "Neutron networks: %s", ex)
             raise
         if app.tag:
             _neutron_net_add_tags(network_id, container_net_id, tags=app.tag)
-            _neutron_net_add_tag(network_id, const.KURYR_EXISTING_NEUTRON_NET)
+            _neutron_net_add_tag(network_id,
+                                 utils.existing_net_tag(container_net_id))
         else:
             network = app.neutron.update_network(
                 neutron_uuid, {'network': {'name': neutron_network_name}})
-            LOG.info(_LI("Updated the network with new name "
-                         "%(neutron_network_name)s successfully: %(network)s"),
+            LOG.info("Updated the network with new name "
+                     "%(neutron_network_name)s successfully: %(network)s",
                      {'neutron_network_name': neutron_network_name,
                       'network': network})
-        LOG.info(_LI("Using existing network %s "
-                     "successfully"), specified_network)
+        LOG.info("Using existing network %s "
+                 "successfully", specified_network)
 
     def _get_existing_neutron_subnets(pool_cidr, network_id):
         cidr = None
@@ -797,7 +865,7 @@ def network_driver_create_network():
 
     # This will add a subnetpool_id(created by kuryr) tag
     # for existing Neutron subnets.
-    if app.tag:
+    if app.tag_ext:
         _add_tag_for_existing_subnet(v4_subnets, v4_pool_id)
         _add_tag_for_existing_subnet(v6_subnets, v6_pool_id)
 
@@ -836,23 +904,23 @@ def network_driver_delete_network():
                                                       tags=app.tag)
     if app.tag:
         existing_network_identifier = neutron_network_identifier + ','
-        existing_network_identifier += const.KURYR_EXISTING_NEUTRON_NET
+        existing_network_identifier += utils.existing_net_tag(container_net_id)
         try:
             existing_networks = _get_networks_by_identifier(
                 existing_network_identifier)
         except n_exceptions.NeutronClientException as ex:
-            LOG.error(_LE("Error happened during listing "
-                          "Neutron networks: %s"), ex)
+            LOG.error("Error happened during listing "
+                      "Neutron networks: %s", ex)
             raise
 
         if existing_networks:
-            LOG.warning(_LW("Network is a pre existing Neutron "
-                            "network, not deleting in Neutron. "
-                            "removing tags: %s"), existing_network_identifier)
+            LOG.warning("Network is a pre existing Neutron "
+                        "network, not deleting in Neutron. "
+                        "removing tags: %s", existing_network_identifier)
             neutron_net_id = existing_networks[0]['id']
             _neutron_net_remove_tags(neutron_net_id, container_net_id)
             _neutron_net_remove_tag(neutron_net_id,
-                                    const.KURYR_EXISTING_NEUTRON_NET)
+                                    utils.existing_net_tag(container_net_id))
             # Delete subnets created by kuryr
             filtered_subnets = _get_subnets_by_attrs(
                 network_id=neutron_net_id)
@@ -862,23 +930,23 @@ def network_driver_delete_network():
                     if str(subnet_name).startswith(const.SUBNET_NAME_PREFIX):
                         app.neutron.delete_subnet(subnet['id'])
                 except n_exceptions.Conflict as ex:
-                    LOG.error(_LE("Subnet %s is in use, "
-                                  "can't be deleted."), subnet['id'])
+                    LOG.error("Subnet %s is in use, "
+                              "can't be deleted.", subnet['id'])
                 except n_exceptions.NeutronClientException as ex:
-                    LOG.error(_LE("Error happened during deleting a "
-                                  "subnet created by kuryr: %s"), ex)
+                    LOG.error("Error happened during deleting a "
+                              "subnet created by kuryr: %s", ex)
             return flask.jsonify(const.SCHEMA['SUCCESS'])
 
     try:
         filtered_networks = _get_networks_by_identifier(
             neutron_network_identifier)
     except n_exceptions.NeutronClientException as ex:
-        LOG.error(_LE("Error happened during listing "
-                      "Neutron networks: %s"), ex)
+        LOG.error("Error happened during listing "
+                  "Neutron networks: %s", ex)
         raise
 
     if not filtered_networks:
-        LOG.warning(_LW("Network with identifier %s cannot be found"),
+        LOG.warning("Network with identifier %s cannot be found",
                     neutron_network_identifier)
     else:
         neutron_network_id = filtered_networks[0]['id']
@@ -904,22 +972,22 @@ def network_driver_delete_network():
                     # and continue to proceed.
                     app.neutron.delete_subnet(subnet['id'])
             except n_exceptions.Conflict as ex:
-                LOG.error(_LE("Subnet, %s, is in use. Network can't "
-                              "be deleted."), subnet['id'])
+                LOG.error("Subnet, %s, is in use. Network can't "
+                          "be deleted.", subnet['id'])
                 raise
             except n_exceptions.NeutronClientException as ex:
-                LOG.error(_LE("Error happened during deleting a "
-                              "Neutron subnets: %s"), ex)
+                LOG.error("Error happened during deleting a "
+                          "Neutron subnets: %s", ex)
                 raise
 
         try:
             app.neutron.delete_network(neutron_network_id)
         except n_exceptions.NeutronClientException as ex:
-            LOG.error(_LE("Error happened during deleting a "
-                          "Neutron network: %s"), ex)
+            LOG.error("Error happened during deleting a "
+                      "Neutron network: %s", ex)
             raise
-        LOG.info(_LI("Deleted the network with ID %s "
-                     "successfully"), neutron_network_id)
+        LOG.info("Deleted the network with ID %s "
+                 "successfully", neutron_network_id)
     return flask.jsonify(const.SCHEMA['SUCCESS'])
 
 
@@ -994,16 +1062,16 @@ def network_driver_create_endpoint():
         except (exceptions.VethCreationFailure,
                 exceptions.BindingNotSupportedFailure) as ex:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Preparing the veth '
-                              'pair was failed: %s.'), ex)
+                LOG.error('Preparing the veth '
+                          'pair was failed: %s.', ex)
         except processutils.ProcessExecutionError:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Could not bind the Neutron port to '
-                              'the veth endpoint.'))
+                LOG.error('Could not bind the Neutron port to '
+                          'the veth endpoint.')
         except (exceptions.KuryrException,
                 n_exceptions.NeutronClientException) as ex:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Failed to set up the interface: %s'), ex)
+                LOG.error('Failed to set up the interface: %s', ex)
 
         if app.vif_plug_is_fatal:
             port_active = _port_active(neutron_port['id'],
@@ -1109,15 +1177,15 @@ def network_driver_delete_endpoint():
                 LOG.error(stderr)
         except processutils.ProcessExecutionError:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Could not unbind the Neutron port from'
-                              'the veth endpoint.'))
+                LOG.error('Could not unbind the Neutron port from'
+                          'the veth endpoint.')
         except exceptions.VethDeletionFailure:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Cleaning the veth pair up was failed.'))
+                LOG.error('Cleaning the veth pair up was failed.')
         except (exceptions.KuryrException,
                 n_exceptions.NeutronClientException) as ex:
             with excutils.save_and_reraise_exception():
-                LOG.error(_LE('Error while removing the interface: %s'), ex)
+                LOG.error('Error while removing the interface: %s', ex)
 
     return flask.jsonify(const.SCHEMA['SUCCESS'])
 
@@ -1360,7 +1428,6 @@ def ipam_request_pool():
     subnet_cidr = ''
     pool_name = ''
     pool_id = ''
-    pools = []
     options = json_data.get('Options')
     if options:
         if v6:
@@ -1370,7 +1437,6 @@ def ipam_request_pool():
             pool_name = options.get(const.NEUTRON_POOL_NAME_OPTION)
             pool_id = options.get(const.NEUTRON_POOL_UUID_OPTION)
     if requested_pool:
-        LOG.info(_LI("Creating subnetpool with the given pool CIDR"))
         if requested_subpool:
             cidr = ipaddress.ip_network(six.text_type(requested_subpool))
         else:
@@ -1378,25 +1444,11 @@ def ipam_request_pool():
         subnet_cidr = six.text_type(cidr)
         subnets_by_cidr = _get_subnets_by_attrs(cidr=subnet_cidr)
         if len(subnets_by_cidr):
-            LOG.warning(_LW("There is already existing subnet for the "
-                            "same cidr. Please check and specify pool name "
-                            "in Options."))
+            LOG.warning("There is already existing subnet for the "
+                        "same cidr. Please check and specify pool name "
+                        "in Options.")
         if not pool_name and not pool_id:
-            pool_name = lib_utils.get_neutron_subnetpool_name(subnet_cidr)
-            pools = _get_subnetpools_by_attrs(name=pool_name)
-            if len(pools):
-                raise exceptions.KuryrException(
-                       "Another pool with same cidr exist. ipam and network"
-                       " options not used to pass pool name")
-
-            new_subnetpool = {
-                'name': pool_name,
-                'default_prefixlen': cidr.prefixlen,
-                'prefixes': [subnet_cidr]}
-            created_subnetpool_response = app.neutron.create_subnetpool(
-                {'subnetpool': new_subnetpool})
-            pool = created_subnetpool_response['subnetpool']
-            pool_id = pool['id']
+            pool_id = _create_kuryr_subnetpool(subnet_cidr)['id']
         else:
             if pool_id:
                 existing_pools = _get_subnetpools_by_attrs(id=pool_id)
@@ -1406,9 +1458,18 @@ def ipam_request_pool():
                 raise exceptions.KuryrException(
                     ("Specified subnetpool id/name({0}) does not "
                     "exist.").format(pool_id or pool_name))
+
             pool_id = existing_pools[0]['id']
-            LOG.info(_LI("Using existing Neutron subnetpool %s successfully"),
-                     pool_id)
+            if app.tag_ext:
+                _neutron_subnetpool_add_tag(
+                    pool_id, const.KURYR_EXISTING_NEUTRON_SUBNETPOOL)
+            prefixes = existing_pools[0]['prefixes']
+            pool_cidr = ipaddress.ip_network(six.text_type(prefixes[0]))
+            if pool_cidr == cidr:
+                LOG.info("Using existing Neutron subnetpool %s successfully",
+                         pool_id)
+            else:
+                pool_id = _create_kuryr_subnetpool(subnet_cidr)['id']
     else:
         if v6:
             default_pool_list = SUBNET_POOLS_V6
@@ -1488,9 +1549,9 @@ def ipam_request_address():
                     if pool_id in tmp_subnet.get('tags'):
                         subnet = tmp_subnet
                 else:
-                    LOG.warning(_LW("subnetpool tag for Neutron "
-                                    "subnet %s is missing, cannot "
-                                    "gets the correct subnet."),
+                    LOG.warning("subnetpool tag for Neutron "
+                                "subnet %s is missing, cannot "
+                                "gets the correct subnet.",
                                 tmp_subnet['id'])
         elif len(subnets_by_cidr) == 1:
             subnet = subnets_by_cidr[0]
@@ -1557,8 +1618,8 @@ def ipam_request_address():
                 allocated_address = '{}/{}'.format(allocated_address,
                                                    subnet_cidr.prefixlen)
             except n_exceptions.NeutronClientException as ex:
-                LOG.error(_LE("Error happened during ip allocation on "
-                              "Neutron side: %s"), ex)
+                LOG.error("Error happened during ip allocation on "
+                          "Neutron side: %s", ex)
                 raise
     else:
         # Auxiliary address or gw_address is received at network creation time.
@@ -1610,7 +1671,15 @@ def ipam_release_pool():
     pools = _get_subnetpools_by_attrs(id=pool_id)
     if pools:
         pool_name = pools[0]['name']
-        if not pool_name.startswith(cfg.CONF.subnetpool_name_prefix):
+        if app.tag_ext:
+            tags = pools[0].get('tags', [])
+            if const.KURYR_EXISTING_NEUTRON_SUBNETPOOL in tags:
+                _neutron_subnetpool_remove_tag(
+                    pool_id, const.KURYR_EXISTING_NEUTRON_SUBNETPOOL)
+                LOG.debug('Skip the cleanup since this is an existing Neutron '
+                          'subnetpool.')
+                return flask.jsonify(const.SCHEMA['SUCCESS'])
+        elif not pool_name.startswith(cfg.CONF.subnetpool_name_prefix):
             LOG.debug('Skip the cleanup since this is an existing Neutron '
                       'subnetpool.')
             return flask.jsonify(const.SCHEMA['SUCCESS'])
@@ -1618,11 +1687,11 @@ def ipam_release_pool():
     try:
         app.neutron.delete_subnetpool(pool_id)
     except n_exceptions.Conflict as ex:
-        LOG.info(_LI("The subnetpool with ID %s is still in use."
-                     " It can't be deleted for now."), pool_id)
+        LOG.info("The subnetpool with ID %s is still in use."
+                 " It can't be deleted for now.", pool_id)
     except n_exceptions.NeutronClientException as ex:
-        LOG.error(_LE("Error happened during deleting a "
-                      "Neutron subnetpool: %s"), ex)
+        LOG.error("Error happened during deleting a "
+                  "Neutron subnetpool: %s", ex)
         raise
 
     return flask.jsonify(const.SCHEMA['SUCCESS'])
@@ -1661,7 +1730,7 @@ def ipam_release_address():
         subnet_cidr = six.text_type(_get_cidr_from_subnetpool(id=pool_id)[0])
         subnets = _get_subnets_by_attrs(cidr=subnet_cidr)
     if not len(subnets):
-        LOG.info(_LI("Subnet already deleted."))
+        LOG.info("Subnet already deleted.")
         return flask.jsonify(const.SCHEMA['SUCCESS'])
 
     iface = ipaddress.ip_interface(six.text_type(rel_address))
@@ -1684,8 +1753,8 @@ def ipam_release_address():
                 _neutron_port_remove_tag(port['id'],
                                          const.KURYR_EXISTING_NEUTRON_PORT)
     except n_exceptions.NeutronClientException as ex:
-        LOG.error(_LE("Error happened while fetching "
-                      "and deleting port, %s"), ex)
+        LOG.error("Error happened while fetching "
+                  "and deleting port, %s", ex)
         raise
 
     return flask.jsonify(const.SCHEMA['SUCCESS'])
