@@ -112,11 +112,17 @@ class TestKuryrIpam(base.TestKuryrBase):
         decoded_json = jsonutils.loads(response.data)
         self.assertEqual(fake_kuryr_subnetpool_id, decoded_json['PoolID'])
 
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.add_tag')
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnetpools')
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnets')
-    @ddt.data((FAKE_IP4_CIDR), (FAKE_IP6_CIDR))
+    @mock.patch('kuryr_libnetwork.controllers.app')
+    @ddt.data((True, FAKE_IP4_CIDR), (True, FAKE_IP6_CIDR),
+              (False, FAKE_IP4_CIDR), (False, FAKE_IP6_CIDR))
+    @ddt.unpack
     def test_ipam_driver_request_pool_with_pool_name_option(self,
-            pool_cidr, mock_list_subnets, mock_list_subnetpools):
+            use_tag_ext, pool_cidr, mock_app, mock_list_subnets,
+            mock_list_subnetpools, mock_add_tag):
+        mock_app.tag_ext = use_tag_ext
         fake_subnet = {"subnets": []}
         mock_list_subnets.return_value = fake_subnet
 
@@ -149,14 +155,26 @@ class TestKuryrIpam(base.TestKuryrBase):
 
         self.assertEqual(200, response.status_code)
         mock_list_subnets.assert_called_with(cidr=pool_cidr)
+        if mock_app.tag_ext:
+            mock_add_tag.assert_called_once_with(
+                'subnetpools', fake_kuryr_subnetpool_id,
+                const.KURYR_EXISTING_NEUTRON_SUBNETPOOL)
+        else:
+            mock_add_tag.assert_not_called()
         decoded_json = jsonutils.loads(response.data)
         self.assertEqual(fake_kuryr_subnetpool_id, decoded_json['PoolID'])
 
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.add_tag')
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnetpools')
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnets')
-    @ddt.data((FAKE_IP4_CIDR), (FAKE_IP6_CIDR))
+    @mock.patch('kuryr_libnetwork.controllers.app')
+    @ddt.data((True, FAKE_IP4_CIDR), (True, FAKE_IP6_CIDR),
+              (False, FAKE_IP4_CIDR), (False, FAKE_IP6_CIDR))
+    @ddt.unpack
     def test_ipam_driver_request_pool_with_pool_id_option(self,
-            pool_cidr, mock_list_subnets, mock_list_subnetpools):
+            use_tag_ext, pool_cidr, mock_app, mock_list_subnets,
+            mock_list_subnetpools, mock_add_tag):
+        mock_app.tag_ext = use_tag_ext
         fake_subnet = {"subnets": []}
         mock_list_subnets.return_value = fake_subnet
 
@@ -186,6 +204,100 @@ class TestKuryrIpam(base.TestKuryrBase):
 
         self.assertEqual(200, response.status_code)
         mock_list_subnets.assert_called_with(cidr=pool_cidr)
+        if mock_app.tag_ext:
+            mock_add_tag.assert_called_once_with(
+                'subnetpools', fake_kuryr_subnetpool_id,
+                const.KURYR_EXISTING_NEUTRON_SUBNETPOOL)
+        else:
+            mock_add_tag.assert_not_called()
+        decoded_json = jsonutils.loads(response.data)
+        self.assertEqual(fake_kuryr_subnetpool_id, decoded_json['PoolID'])
+
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.add_tag')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.create_subnetpool')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnetpools')
+    @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnets')
+    @mock.patch('kuryr_libnetwork.controllers.app')
+    @ddt.data((True, FAKE_IP4_CIDR), (True, FAKE_IP6_CIDR),
+              (False, FAKE_IP4_CIDR), (False, FAKE_IP6_CIDR))
+    @ddt.unpack
+    def test_ipam_driver_request_pool_with_unmatched_cidr(self,
+            use_tag_ext, pool_cidr, mock_app, mock_list_subnets,
+            mock_list_subnetpools, mock_create_subnetpool, mock_add_tag):
+        mock_app.tag_ext = use_tag_ext
+        fake_subnet = {"subnets": []}
+        mock_list_subnets.return_value = fake_subnet
+        subnet_ip4_cidr = '10.0.0.0/24'
+        subnet_ip6_cidr = 'fe80::/68'
+        if pool_cidr == FAKE_IP4_CIDR:
+            subnet_cidr = subnet_ip4_cidr
+        else:
+            subnet_cidr = subnet_ip6_cidr
+        pool_name = lib_utils.get_neutron_subnetpool_name(subnet_cidr)
+        prefixlen = ipaddress.ip_network(six.text_type(subnet_cidr)).prefixlen
+        new_subnetpool = {
+            'name': pool_name,
+            'default_prefixlen': prefixlen,
+            'prefixes': [subnet_cidr]}
+
+        fake_kuryr_subnetpool_id = uuidutils.generate_uuid()
+        fake_existing_subnetpool_id = uuidutils.generate_uuid()
+        if pool_cidr == FAKE_IP4_CIDR:
+            kuryr_subnetpools = self._get_fake_v4_subnetpools(
+                fake_kuryr_subnetpool_id, prefixes=[subnet_ip4_cidr])
+            existing_subnetpools = self._get_fake_v4_subnetpools(
+                fake_existing_subnetpool_id, prefixes=[pool_cidr])
+            options = {
+                const.NEUTRON_POOL_UUID_OPTION: fake_existing_subnetpool_id}
+        else:
+            kuryr_subnetpools = self._get_fake_v6_subnetpools(
+                fake_kuryr_subnetpool_id, prefixes=[subnet_ip6_cidr])
+            existing_subnetpools = self._get_fake_v6_subnetpools(
+                fake_existing_subnetpool_id, prefixes=[pool_cidr])
+            options = {
+                const.NEUTRON_V6_POOL_UUID_OPTION: fake_existing_subnetpool_id}
+        mock_list_subnetpools.side_effect = [
+            existing_subnetpools,
+            {'subnetpools': []}
+        ]
+
+        fake_subnetpool_response = {
+            'subnetpool': kuryr_subnetpools['subnetpools'][0]
+        }
+        mock_create_subnetpool.return_value = fake_subnetpool_response
+
+        if pool_cidr == FAKE_IP4_CIDR:
+            subnet_cidr = subnet_ip4_cidr
+            fake_request = {
+                'AddressSpace': '',
+                'Pool': subnet_cidr,
+                'SubPool': subnet_cidr,
+                'Options': options,
+                'V6': False,
+            }
+        else:
+            subnet_cidr = subnet_ip6_cidr
+            fake_request = {
+                'AddressSpace': '',
+                'Pool': subnet_cidr,
+                'SubPool': subnet_cidr,
+                'Options': options,
+                'V6': True,
+            }
+        response = self.app.post('/IpamDriver.RequestPool',
+                                content_type='application/json',
+                                data=jsonutils.dumps(fake_request))
+
+        self.assertEqual(200, response.status_code)
+        mock_list_subnets.assert_called_with(cidr=subnet_cidr)
+        mock_create_subnetpool.assert_called_with(
+            {'subnetpool': new_subnetpool})
+        if mock_app.tag_ext:
+            mock_add_tag.assert_called_once_with(
+                'subnetpools', fake_existing_subnetpool_id,
+                const.KURYR_EXISTING_NEUTRON_SUBNETPOOL)
+        else:
+            mock_add_tag.assert_not_called()
         decoded_json = jsonutils.loads(response.data)
         self.assertEqual(fake_kuryr_subnetpool_id, decoded_json['PoolID'])
 
@@ -284,9 +396,12 @@ class TestKuryrIpam(base.TestKuryrBase):
 
         fake_kuryr_subnetpool_id = uuidutils.generate_uuid()
         fake_subnetpool_name = 'fake_pool_name'
+        fake_tags = []
+        if mock_app.tag_ext:
+            fake_tags.append(const.KURYR_EXISTING_NEUTRON_SUBNETPOOL)
         kuryr_subnetpools = self._get_fake_v4_subnetpools(
             fake_kuryr_subnetpool_id, prefixes=[FAKE_IP4_CIDR],
-            name=fake_subnetpool_name)
+            name=fake_subnetpool_name, tags=fake_tags)
         mock_list_subnetpools.return_value = kuryr_subnetpools
 
         docker_endpoint_id = lib_utils.get_hash()
@@ -316,9 +431,12 @@ class TestKuryrIpam(base.TestKuryrBase):
                 id=fake_kuryr_subnetpool_id)
             mock_list_subnets.assert_called_with(
                 cidr=FAKE_IP4_CIDR)
-            mock_remove_tag.assert_called_with('subnets',
-                                               subnet_v4_id,
-                                               fake_kuryr_subnetpool_id)
+            mock_remove_tag.assert_any_call('subnets',
+                                            subnet_v4_id,
+                                            fake_kuryr_subnetpool_id)
+            mock_remove_tag.assert_any_call(
+                'subnetpools', fake_kuryr_subnetpool_id,
+                const.KURYR_EXISTING_NEUTRON_SUBNETPOOL)
 
     @mock.patch('kuryr_libnetwork.controllers._neutron_port_add_tag')
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.create_port')
@@ -769,9 +887,9 @@ class TestKuryrIpam(base.TestKuryrBase):
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.list_subnetpools')
     @mock.patch('kuryr_libnetwork.controllers.app')
     @ddt.data((False), (True))
-    def test_ipam_driver_request_address_overlapping_cidr_in_kuryr(self,
-            use_tag_ext, mock_app, mock_list_subnetpools, mock_list_subnets,
-        mock_create_port, mock_port_add_tag):
+    def test_ipam_driver_request_address_overlapping_cidr_in_kuryr(
+            self, use_tag_ext, mock_app, mock_list_subnetpools,
+            mock_list_subnets, mock_create_port, mock_port_add_tag):
         mock_app.tag_ext = use_tag_ext
         # faking list_subnetpools
         fake_kuryr_subnetpool_id = uuidutils.generate_uuid()
@@ -925,7 +1043,7 @@ class TestKuryrIpam(base.TestKuryrBase):
         self.assertIn('Err', decoded_json)
         err_message = ("Requested gateway {0} does not match with "
                        "gateway {1} in existed network.").format(
-                       '10.0.0.5', '10.0.0.1')
+                           '10.0.0.5', '10.0.0.1')
         self.assertEqual({'Err': err_message}, decoded_json)
 
     @mock.patch('kuryr_libnetwork.controllers.app.neutron.delete_port')
